% Computes the dynamics of a double pendulum consisting of two masses
% connected by rigid light rods. Ignore any energy losses such as air
% resistance and hinge friction. a1, a2 are the angles of each pendulum
% bob's rod measured anticlockwise from the vertical.
function double_pendulum
global d

%%%% INPUTS %%%%

m1=1; m2=4; %Bob masses /kg
l1=3; l2=1; %Rigid connecting rod lengths (negligible mass) /m
g=9.81; %Gravitational acceleration /ms^-2
T=20; %Duration of solution set /s
dt=0.005;  %Timestep for Runge-Kutta numeric method
fsize = 14;  %Fontsize for graphs

%Initial bob rod angles in degrees.
%(Measured anticlockise from the vertical).
a1_init = 160; a2_init = 160;

%Initial bob angular velocities /degrees per second
w1_init= -180; w2_init = 360;

%Store parameters required for angular acceleration computation
d.m1=m1; d.m2=m2; d.l1=l1; d.l2=l2; d.g=g;

%Plot ... vs time graphs
plot_graphs = 0;

%AVI movie option defaults (getframe=0 is much slower but better quality)
d.movie = 0; d.getframe = 0; FPS = 30; speed = 1.0;

%

%%%% COMPUTE TRAJECTORY %%%%

%Determine evaluation times for simulation /s
t = 0 : dt : T; N = length(t);

%Determine trajectory evaluations per video frame
nf = round( speed/(FPS*dt) ); if nf==0; nf = 1; end

%Convert initial angles and angular velocities to radians
a1(1) = a1_init*pi/180; a2(1) = a2_init*pi/180;
w1(1) = w1_init*pi/180; w2(1) = w2_init*pi/180;

%Step through time in steps dt and use the Runge Kutta method to integrate
%the four coupled differential equations which relate a1,a2,w1,w2.
A1 = a1; A2 = a2; W1 = w1; W2 = w2;
for n=1:N-1
    
    %Step1
    k1_a1 = w1; k1_a2 = w2;
    k1_w1 = dw1_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    k1_w2 = dw2_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    a1 = A1(n) + 0.5*k1_a1*dt; a2 = A2(n) + 0.5*k1_a2*dt;
    w1 = W1(n) + 0.5*k1_w1*dt; w2 = W2(n) + 0.5*k1_w2*dt;
    
    %Step 2
    k2_a1 = w1; k2_a2 = w2;
    k2_w1 = dw1_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    k2_w2 = dw2_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    a1 = A1(n) + 0.5*k2_a1*dt; a2 = A2(n) + 0.5*k2_a2*dt;
    w1 = W1(n) + 0.5*k2_w1*dt; w2 = W2(n) + 0.5*k2_w2*dt;
    
    %Step 3
    k3_a1 = w1; k3_a2 = w2;
    k3_w1 = dw1_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    k3_w2 = dw2_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    a1 = A1(n) + k3_a1*dt; a2 = A2(n) + k3_a2*dt;
    w1 = W1(n) + k3_w1*dt; w2 = W2(n) + k3_w2*dt;
    
    %Step 4
    k4_a1 = w1; k4_a2 = w2;
    k4_w1 = dw1_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    k4_w2 = dw2_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
    
    %Update angles and angular velocities
    a1 = A1(n) + (dt/6)*( k1_a1 + 2*k2_a1 + 2*k3_a1 + k4_a1 );
    a2 = A2(n) + (dt/6)*( k1_a2 + 2*k2_a2 + 2*k3_a2 + k4_a2 );
    w1 = W1(n) + (dt/6)*( k1_w1 + 2*k2_w1 + 2*k3_w1 + k4_w1 );
    w2 = W2(n) + (dt/6)*( k1_w2 + 2*k2_w2 + 2*k3_w2 + k4_w2 );
    A1(n+1) = a1; A2(n+1) = a2;
    W1(n+1) = w1; W2(n+1) = w2;
end

%Reassign angle and angular velocity variables for brevity
a1 = A1; a2 = A2; w1 = W1; w2 = W2; clear A1 A2 W1 W2

%Compute angular accelerations
w1dot = dw1_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );
w2dot = dw2_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g );

%Wrap angles to -pi...pi
a1 = anglewrap(a1); a2 = anglewrap(a2);

%Compute pendulum bob x,y coordinates /m
x1 = l1*sin(a1);
y1 = -l1*cos(a1);
x2 = l1*sin(a1) + l2*sin(a2);
y2 = -l1*cos(a1) - l2*cos(a2);

%Compute pendulum bob x,y velocities /ms^-1
vx1 = l1*cos(a1).*w1;
vy1 = l1*sin(a1).*w1;
vx2 = l1*cos(a1).*w1 + l2*cos(a2).*w2;
vy2 = l1*sin(a1).*w1 + l2*sin(a2).*w2;

%Compute kinetic energy /J
ke1 = 0.5*m1*( vx1.^2 + vy1.^2 ); ke2 = 0.5*m2*( vx2.^2 + vy2.^2 ) ;

%Compute potential energy /J
pe1 = m1*g*y1; pe2 = m2*g*y2;

%Compute angular momenta /kg * m^2 * s^-1
%projected onto the Cartesian z direction
j1 = angular_momentum( m1, x1, y1 ,vx1, vy1 );
j2 = angular_momentum( m2, x2, y2 ,vx2, vy2 );

%Save data
save dpendulum

%

%%%% PLOT GRAPHS & ANIMATE TRAJECTORY %%%%

%Diagnostic .... vs t graphs
if plot_graphs == 1
    dpendulum_graphs;
end

%Generate figure and axes for first frame of animation ...
figure('color',[1 1 1],'name','Double Pendulum','renderer','opengl',...
    'units','pixels','numbertitle','off','position',get(0,'screensize'),...
    'KeyPressFcn',@keypress);
set(gca,'nextplot','add')

%Generate coordinates of spheres for pendulum bob positions
[xs,ys,zs]=sphere(100);
xs=xs*0.3; ys=ys*0.3; zs=zs*0.3;

%Generate coordinates of a unit circle
[xc,yc]=circle(200);

%Plot black sphere fixed point which is the hinge of bob 1
fp=surf( xs,ys,zs ); set(fp,'facecolor',[0 0 0],'edgecolor','none');

%Plot blue sphere for bob 1
bob1=surf( xs + x1(1),ys +y1(1), zs );
set(bob1,'facecolor',[0 0 1],'edgecolor','none');

%Plot red sphere for bob 2
bob2=surf( xs + x2(1),ys +y2(1), zs );
set(bob2,'facecolor',[1 0 0],'edgecolor','none');

%Plot lines joining fixed point to bobs 1 and 2
line12 = plot( [0,x1(1),x2(1)],[0,y1(1),y2(1)],'k' );

%Plot circles defining possible positions of bobs 1 and 2
circle1 = plot( xc*l1,yc*l1,'color',[0,0,1] );
circle2 = plot( xc*l2 + x1(1),yc*l2 + y1(1),'color',[1,0,0] );

%Plot trajectory of bob 2
bob2_traj = plot( x2(1),y2(1),'g' );

%Set 2D view and axis limits
view(2); axis equal; xlim([-l1-2*l2,l1+2*l2]);
ylim([-l1-2*l2,l1+2*l2]); zlim([-l1-2*l2,l1+2*l2]);
axis off; camlight; lighting phong

%Set title, with time information
tit = title({'\bfDouble pendulum',...
    ['\rmm1=',num2str(m1),'kg  m2=',num2str(m2),...
    'kg  l1=',num2str(l1),' metres  l2=',num2str(l2),' metres'],...
    ['time =',num2str(floor(t(n))),' s']},'fontsize',fsize);

%Open AVI file if option set
if d.movie == 1
    d.V = VideoWriter('dpendulum trajectory.avi');
    d.V.FrameRate = 30; open(d.V); d.frame = 1;
    disp(' '); disp(' Generating video frames ... ')
    d.t0 = tic;
end

%Animate ...
d.stop = 0; d.n = 1;  d.frame = 0; d.V = []; frames = 1:nf:N;
while d.stop ==0
    
    %Increment frame number
    d.frame = d.frame + 1;
    if d.frame > length(frames)
        d.frame = 1;
    end
    
    %Determine trajectory index
    n = frames( d.frame );
    
    %Update plot elements
    set(bob1,'xdata',xs + x1(n),'ydata',ys +y1(n) );
    set(bob2,'xdata',xs + x2(n),'ydata',ys +y2(n) );
    set( line12, 'xdata', [0,x1(n),x2(n)], 'ydata',[0,y1(n),y2(n)] );
    set( circle1, 'xdata',xc*l1,'ydata',yc*l1 );
    set( circle2, 'xdata',xc*l2 + x1(n),'ydata',yc*l2 + y1(n) );
    set( bob2_traj,'xdata',x2(1:n),'ydata',y2(1:n) )
    set( tit, 'string',{'\bfDouble pendulum',...
        ['\rmm1=',num2str(m1),'kg  m2=',num2str(m2),...
        'kg  l1=',num2str(l1),' metres  l2=',num2str(l2),' metres'],...
        ['time =',num2str(floor(t(n))),' s']} );
    
    %Add to movie frame if option set
    if d.movie == 1
        %Add video frame
        if d.getframe==1
            %Quicker but lower quality method
            I = getframe(gcf); I = I.cdata;
        else
            print( gcf, 'frame.png','-r300','-dpng' );
            I = imread( 'frame.png' ); delete('frame.png');
        end
        writeVideo( d.V,uint8(I) );
        disp([' Frame ',leadzero(4,d.frame),' of ',num2str( length(frames) ),' added to video ...'] );
    else
        close(d.V); d.V = []; d.movie = 0;
    end
    
    %End movie if end of trajectory is attained
    if (n==N) && (d.movie==1) 
        d.movie=0; 
        disp( ['Movie generated in ',num2str(toc(d.t0)/60),' minutes.'] )
    end
    
    %Flush graphics buffer to force screen update
    drawnow;
end
close(d.V); close(gcf);

%%

%Rate of change of angular velocity of bob 1
function w1dot = dw1_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g )
D = a2-a1;
w1dot  = ( m2*l1*w1.*w1.*sin(D).*cos(D)+m2*g*sin(a2).*cos(D) +...
    m2*l2*w2.*w2.*sin(D)-(m1+m2)*g*sin(a1) )./...
    ( (m1+m2)*l1 - m2*l1*(cos(D).^2) ) ;

%%

%Rate of change of angular velocity of bob2
function w2dot = dw2_by_dt( a1,a2,w1,w2,m1,m2,l1,l2,g )
D = a2-a1;
w2dot  = ( -m2*l2*w2.*w2.*sin(D).*cos(D) +...
    (m1+m2)*( g*sin(a1).*cos(D) - l1*w1.*w1.*sin(D) - g*sin(a2) ) )./...
    ( (m1+m2)*l2 - m2*l2*(cos(D).^2) ) ;

%%

%Angular momentum (in the z direction)
function j = angular_momentum( m, x, y ,vx, vy )
o = ones(size(vx));
j = m*dot( [0*o; 0*o; o], cross( [ x; y; o], [vx;vy;o] ) );

%%

%x,y coordinates of a unit circle
function [xc,yc]=circle(N)
a=linspace(0,2*pi,N); xc=cos(a); yc=sin(a);

%%

%Wrap angle to [-pi,pi]
function y = anglewrap(x)
y = atan2( sin(x), cos(x) );

%%

%Leading zero function
function s=leadzero(Nz,z)
fmt=num2str(Nz,'%%0%dg'); s=sprintf(fmt,z);

%%

% Key press handler
function keypress( fig,evnt )
global d
if strcmp(get(gcf,'currentkey'),'q')==1
    %Quitting procedure
    d.stop = 1;
elseif strcmp(get(gcf,'currentkey'),'p')==1
    %Print screen to PNG file
    if d.movie==0
        print( gcf,'dpendulum screenshot.png','-dpng','-r300' );
    end
elseif strcmp(get(gcf,'currentkey'),'m')==1
    %AVI movie toggle
    if d.movie==1
        d.movie=0; d.frame = 0;
        disp(' Video generation cancelled.')
    else
        d.movie = 1; d.n = 1; d.t0 = tic;
        d.V = VideoWriter('dpendulum trajectory.avi');
        d.V.FrameRate = 30; open(d.V); d.frame = 0;
        disp(' '); disp(' Generating video frames ... ')
    end
end

%%

%Plot diagnostic graphs
function dpendulum_graphs
load dpendulum
figure('color',[1 1 1],'name','Double Pendulum')

%Plot Poincare diagrams. Plot bob angles against bob angular velocities
set(gca,'fontsize',fsize,'nextplot','add'); 
p1=plot(a1,w1,'b.'); set(p1,'markersize',2,'markerfacecolor',[0,1,0])
p2=plot(a2,w2,'r.'); set(p2,'markersize',2,'markerfacecolor',[0,0,0])
xlabel('Angle /rad'); ylabel('Angular velocity /rad s^{-1}')
title('Poincare diagram: bob1 is blue, bob2 is red')
axis tight; grid on; box on
print(gcf, 'dpendulum poincare.png','-r300','-dpng' ); clf;

%

%Plot bob rod angles from the vertical /degrees vs time /seconds
subplot(2,1,1,'fontsize',fsize)
plot(t,a1,'b'); ylim([-pi,pi]); axis tight; grid on
xlabel('Time /s'); ylabel('Angle /rad');
title('Pendulum bob rod 1 angle from vertical','fontsize',fsize)

subplot(2,1,2,'fontsize',fsize)
plot(t,a2,'r'); ylim([-pi,pi]); axis tight; grid on
xlabel('Time /s'); ylabel('Angle /rad')
title('Pendulum bob rod 2 angle from vertical')
print(gcf, 'dpendulum t vs angle.png','-r300','-dpng' ); clf;

%

%Plot bob Cartesian x and y coordinates /metres vs time /s
subplot(2,1,1,'fontsize',fsize)
plot(t,x1,'b',t,x2,'r'); axis tight; grid on;
xlabel('Time /s'); ylabel('x /metres')
legend({'bob1','bob2'}); title('Pendulum bob x positions');

subplot(2,1,2,'fontsize',fsize)
plot(t,y1,'b',t,y2,'r'); axis tight; grid on;
xlabel('Time /s'); ylabel('y /metres')
legend({'bob1','bob2'}); title('Pendulum bob y positions')
print(gcf, 'dpendulum xyt.png','-r300','-dpng' ); clf;

%Plot Cartesian x,y trajectories of pendulum bobs
set(gca,'fontsize',fsize,'nextplot','add');
p1=plot(x1,y1,'b.'); set(p1,'markersize',2,'markerfacecolor',[0,1,0])
p2=plot(x2,y2,'r.'); set(p2,'markersize',2,'markerfacecolor',[0,0,0])
xlabel('x /metres'); ylabel('y /metres')
title('Pendulum bob x,y trajectories. Bob1 is blue, Bob2 is red')
axis equal; axis tight; grid on; box on;
print(gcf, 'dpendulum xy.png','-r300','-dpng' ); clf;

%

%Plot kinetic and potential energies
k = 0.8;
subplot(2,2,1,'fontsize',fsize*k);
plot(t,ke1,'b',t,pe1,'b--'); grid on; 
xlabel('Time /s'); ylabel('Energy /joules')
title('Pendulum bob 1 kinetic, potential energy'); 
legend('Kinetic','Potential'); axis tight

subplot(2,2,2,'fontsize',fsize*k);
plot(t,ke2,'r',t,pe2,'r--')
xlabel('Time /s'); ylabel('Energy /joules')
title('Pendulum bob 2 kinetic, potential energy')
legend('Kinetic','Potential'); grid on; axis tight

subplot(2,2,3,'fontsize',fsize*k);
plot(t,ke1+pe1,'b',t,ke2+pe2,'r')
xlabel('Time /s'); ylabel('Energy /joules')
title('Total energies, per bob')
legend('Bob 1','Bob 2'); axis tight; grid on

subplot(2,2,4,'fontsize',fsize*k);
plot(t,ke1+pe1+ke2+pe2,'m')
ylim([min(min([pe1,pe2])),max(max([ke1,ke2]))]);
xlabel('Time /s'); ylabel('Energy /joules')
title('Total energy of system'); grid on
print(gcf, 'dpendulum energy.png','-r300','-dpng' ); clf;

%

%Plot angular velocity /rads^-1 and angular momentum projected onto Z axis
subplot(2,1,1,'fontsize',fsize);
plot(t,w1,'b',t,w2,'r')
xlabel('Time /s'); ylabel('Angular velocities / rad s^{-1}')
title('Angular velocities'); legend({'Bob 1','Bob 2'}); grid on

subplot(2,1,2,'fontsize',fsize)
plot(t,j1,t,j2,t,j1+j2);
xlabel('Time /s'); ylabel('Angular momentum /kgm^2s^{-1}')
title('Angular momentum'); legend('Bob 1','Bob 2','Total')
print(gcf, 'dpendulum adot & J.png','-r300','-dpng' ); clf;

%

%Plot Cartesian velocities /ms^-1
subplot(2,1,1,'fontsize',fsize)
plot(t,vx1,'b',t,vy1,'m')
xlabel('Time /s'); ylabel('Velocity /ms^{-1}')
title('Pendulum bob 1 x and y velocities')
legend({'vx','vy'}); axis tight; grid on

subplot(2,1,2,'fontsize',fsize)
plot(t,vx2,'r',t,vy2,'g')
xlabel('Time /s'); ylabel('Velocity /ms^{-1}')
title('Pendulum bob 2 x and y velocities')
legend({'vx','vy'}); axis tight; grid on
print(gcf, 'dpendulum vxy.png','-r300','-dpng' ); close(gcf);

%%

% Creates a legend, without the need to refer to a plot, or a set of plot
% handles.
function h=thelegend(strings,chars)
L=length(strings);  %Find length of strings

%Plot characters at the origin
for n=1:L; ph(n)=plot(0,0,chars{n}); hold on; end
h=legend(ph,strings); %Create legend
delete(ph); %Delete origin characters

%End of code