% Plots the 'strange attractor' which represents long-term trajectories
% associated with x,y,z coordinates which obey the coupled differential
% equations analysed by Edward Lorenz.
function lorenz

%Lorenz parameters
s = 10;  r = 28;  b = 8/3;

%Range of initial conditions
x0min = -1; x0max = 1;
y0min = -1; y0max = 1;
z0min = -1; z0max = 1;

%Define timestep
dt = 0.005;

%Number of trajectory points prior to plotting
N_before = 10;

%Number of trajectory points to plot
N = 10000;

%Number of random x,y,z initial conditions
M = 10;

%Font size and linewidth
fsize = 16; lwidth = 0.05;

%%

%Define random initial conditions between [-1,1] for x,y,z
x0 = x0min + (x0max-x0min)*rand(1,M);
y0 = y0min + (y0max-y0min)*rand(1,M);
z0 = z0min + (z0max-z0min)*rand(1,M);

%Determine x,y,z vs t Lorenz attractor trajectory
[x,y,z,t] = lorenz_xyz( x0,y0,z0, dt, s, r, b, N, N_before );

%Plot trajectories
figure('name','lorenz','color',[1 1 1],'renderer','opengl',...
    'units','normalized','position',[0 0 1 1]);
plot3(x,y,z,'-','linewidth',lwidth);
axis equal; axis vis3d;
xlabel('x','fontsize',fsize); ylabel('y','fontsize',fsize)
zlabel('z','fontsize',fsize)
set(gca,'fontsize',fsize);
title('Lorenz attractor')
view([50,32]); grid on; box on
print(gcf,'lorenz.png','-dpng','-r300');

%%

%Determine x,y,z vs t Lorenz attractor trajectory
function [x,y,z,t] = lorenz_xyz( x0,y0,z0, dt, s, r, b, N, N_before )

%Initialize output trajectories
M = length(x0);
x = zeros(N,M); y = zeros(N,M); z = zeros(N,M); t = zeros(N,M);

%Step through N_before iterations. Use a Runge Kutta method to determine
%the new trajectories
xx = x0; yy = y0; zz = z0;
for n=1:N_before+N
    %x
    k1 = lorenz_xdot( xx,yy,s );
    k2 = lorenz_xdot( xx + 0.5*dt*k1,yy + 0.5*dt*k1,s );
    k3 = lorenz_xdot( xx + 0.5*dt*k2,yy + 0.5*dt*k2,s );
    k4 = lorenz_xdot( xx + dt*k3,yy + dt*k3,s );
    dx = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %y
    k1 = lorenz_ydot( xx,yy,zz,r );
    k2 = lorenz_ydot( xx + 0.5*dt*k1,yy + 0.5*dt*k1,zz + 0.5*dt*k1,r );
    k3 = lorenz_ydot( xx + 0.5*dt*k2,yy + 0.5*dt*k2,zz + 0.5*dt*k2,r );
    k4 = lorenz_ydot( xx + dt*k3,yy + dt*k3,zz + dt*k3,r);
    dy = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %z
    k1 = lorenz_zdot( xx,yy,zz, b );
    k2 = lorenz_zdot( xx + 0.5*dt*k1, yy + 0.5*dt*k1, zz + 0.5*dt*k1, b );
    k3 = lorenz_zdot( xx + 0.5*dt*k2,yy + 0.5*dt*k2, zz + 0.5*dt*k2, b );
    k4 = lorenz_zdot( xx + dt*k3, yy + dt*k3, zz + dt*k3, b );
    dz = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %Update xx,yy,zz
    xx = xx + dx; yy = yy + dy; zz = zz + dz;
    
    %Store if n > N_before
    if n > N_before
        nn = n - N_before;
        x(nn,:) = xx; y(nn,:) = yy; z(nn,:) = zz; t(nn,:) = nn*dt;
    end
end

%%

%Lorenz ODEs
function xdot = lorenz_xdot( x,y,s )
xdot = s*( y - x );
function ydot = lorenz_ydot( x,y,z,r )
ydot = x.*(r-z) - y;
function zdot = lorenz_zdot( x,y,z,b )
zdot = x.*y - b*z;

%End of code