%rossler
% Plots the 'strange attractor' which represents long-term trajectories
% associated with x,y,z coordinates which obey the coupled differential
% equations analysed by Otto Eberhard Rssler.
%
% LAST UPDATED by Andy French June 2017

function rossler

%Rossler parameters
a = 0.2;
b = 0.2;
c = 5.7;

%Alternative inputs
a = 0.1;
b = 0.1;
c = 14;

%Range of initial conditions
x0min = -1;
x0max = 1;
y0min = -1;
y0max = 1;
z0min = -1;
z0max = 1;

%Define timestep
dt = 0.01;

%Number of trajectory points prior to plotting
N_before = 0;

%Number of trajectory points to plot
N = 10000;

%Number of random x,y,z initial conditions
M = 20;

%Fontsize for plots
fsize = 16;

%Marker size for plots
msize = 0.1;

%%

%Define random initial conditions between [-1,1] for x,y,z
x0 = x0min + (x0max-x0min)*rand(1,M);
y0 = y0min + (y0max-y0min)*rand(1,M);
z0 = z0min + (z0max-z0min)*rand(1,M);

%Initialize output trajectories
x = zeros(N,M);
y = zeros(N,M);
z = zeros(N,M);

%

%Step through N_before iterations. Use a Runge Kutta method to determine
%the new trajectories
xx = x0;
yy = y0;
zz = z0;
for n=1:N_before+N
    %x
    k1 = rossler_xdot( yy,zz );
    k2 = rossler_xdot( yy + 0.5*dt*k1,zz + 0.5*dt*k1 );
    k3 = rossler_xdot( yy + 0.5*dt*k2,zz + 0.5*dt*k2 );
    k4 = rossler_xdot( yy + dt*k3,zz + dt*k3 );
    dx = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %y
    k1 = rossler_ydot( xx,yy,a );
    k2 = rossler_ydot( xx + 0.5*dt*k1,yy + 0.5*dt*k1,a );
    k3 = rossler_ydot( xx + 0.5*dt*k2,yy + 0.5*dt*k2,a );
    k4 = rossler_ydot( xx + dt*k3,yy + dt*k3,a );
    dy = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %z
    k1 = rossler_zdot( xx,zz, b,c );
    k2 = rossler_zdot( xx + 0.5*dt*k1,zz + 0.5*dt*k1, b,c );
    k3 = rossler_zdot( xx + 0.5*dt*k2,zz + 0.5*dt*k2, b,c );
    k4 = rossler_zdot( xx + dt*k3,zz + dt*k3, b,c );
    dz = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %Update xx,yy,zz
    xx = xx + dx;
    yy = yy + dy;
    zz = zz + dz;
    
    %Store if n > N_before
    if n > N_before
        nn = n - N_before;
        x(nn,:) = xx;
        y(nn,:) = yy;
        z(nn,:) = zz;
    end
end

%

%Plot trajectories
figure('name','rossler','color',[1 1 1],'renderer','opengl');
plot3(x,y,z,'-','markersize',msize);
axis equal
axis vis3d
xlabel('x','fontsize',fsize)
ylabel('y','fontsize',fsize)
zlabel('z','fontsize',fsize)
set(gca,'fontsize',fsize);
title(['Rossler attractor'])
grid on
box on
print(gcf,'rossler.png','-dpng','-r300');

%%

%Rossler ODEs
function xdot = rossler_xdot( y,z )
xdot = -y - z;
function ydot = rossler_ydot( x,y,a )
ydot = x + a*y;
function zdot = rossler_zdot( x,z,b,c )
zdot = z.*(x-c) + b;

%End of code