%shaw
% Plots the 'strange attractor' which represents long-term trajectories
% associated with x,y,z coordinates which obey the coupled differential
% equations analysed by Edward Lorenz. A set of M closely spaced initial
% points are overlaid upon the Lorenz attractor, and their location after
% every K iterations is plotted. As Shaw et al showed, the points should
% disperse over the entire attractor.
%
% LAST UPDATED by Andy French Nov 2019

function shaw

%Movie or frames?
movie_or_frames = 'movie';

%Update shaw 'droplet' after K iterations, and how many K's
if strcmp( movie_or_frames, 'movie')==1
    K = 10; NK = 500;
    FPS = 10;
else
    K = 200; NK = 40;
end

%Number of random x,y,z initial conditions
M = 5000;

%Range of initial conditions for shaw droplet
delta = 0.01;
x0min = 1 - delta; x0max = 1 + delta;
y0min = 1 - delta; y0max = 1 + delta;
z0min = 1 - delta; z0max = 1 + delta;

%Define timestep
dt = 0.005;

%Number of trajectory points prior to plotting
N_before = 10;

%Lorenz parameters
s = 10;  r = 28;  b = 8/3;

%Font size and marker size for plots
fsize = 16; msize = 5;

%%

%Determine x,y,z vs t Lorenz attractor trajectory underlay
[x,y,z,t] = lorenz_xyz( 1,1,1, dt, s, r, b, 1e5, N_before );

%Define random initial conditions between for x,y,z
x0 = x0min + (x0max-x0min)*rand(1,M);
y0 = y0min + (y0max-y0min)*rand(1,M);
z0 = z0min + (z0max-z0min)*rand(1,M);

%Determine x,y,z vs t Lorenz attractor shaw droplet trajectory
[xd,yd,zd,td] = lorenz_xyz( x0,y0,z0, dt, s, r, b, NK*K, N_before );

%Make directory for images
if strcmp( movie_or_frames, 'movie')==0
    dirname = ['shaw drop ',strrep(datestr(now),':','-')];
    mkdir(dirname);
end

%Plot trajectories
figure('name','lorenz','color',[1 1 1],'renderer','opengl',...
    'units','normalized','position',[0 0 1 1]);
plot3(x,y,z,'g-','linewidth',0.05); hold on
i = 1; s = plot3(xd(i,:),yd(i,:),zd(i,:),'r.','markersize',msize);
axis equal; axis vis3d;
xlabel('x','fontsize',fsize); ylabel('y','fontsize',fsize)
zlabel('z','fontsize',fsize)
set(gca,'fontsize',fsize);
tit = title( ['Lorenz attractor: iteration ',num2str(i)] );
view([50,32]); grid on; box on
if strcmp( movie_or_frames, 'movie')==0
    print( gcf, [dirname,'\Shaw Lorenz i=',num2str(i),'.png'],'-dpng','-r300' );
else
    %Open video file and add first frame
    v = VideoWriter('shaw.avi'); v.FrameRate = FPS; open(v);
    print( gcf, 'shaw.png','-dpng','-r200' );
    I = imread('shaw.png'); writeVideo(v,I);
end

%Overlay shaw droplet x,y,z positions
for n=1:NK
    i = n*K;
    set( s, 'xdata', xd(i,:),'ydata', yd(i,:),'zdata', zd(i,:) );
    set( tit, 'string', ['Lorenz attractor: iteration ',num2str(i)] );
    if strcmp( movie_or_frames, 'movie')==0
        %Print frame to a PNG file
        print( gcf, [dirname,'\Shaw Lorenz i=',num2str(i),'.png'],'-dpng','-r300' );
    else
        %Add frame to movie
        print( gcf, 'shaw.png','-dpng','-r200' );
        I = imread('shaw.png'); writeVideo(v,I);
    end
end
close(gcf);
if strcmp( movie_or_frames, 'movie')==1
    close(v); delete('shaw.png');
end

%%

%Determine x,y,z vs t Lorenz attractor trajectory
function [x,y,z,t] = lorenz_xyz( x0,y0,z0, dt, s, r, b, N, N_before )

%Initialize output trajectories
M = length(x0);
x = zeros(N,M); y = zeros(N,M); z = zeros(N,M); t = zeros(N,M);

%Step through N_before iterations. Use a Runge Kutta method to determine
%the new trajectories
xx = x0; yy = y0; zz = z0;
for n=1:N_before+N
    %x
    k1 = lorenz_xdot( xx,yy,s );
    k2 = lorenz_xdot( xx + 0.5*dt*k1,yy + 0.5*dt*k1,s );
    k3 = lorenz_xdot( xx + 0.5*dt*k2,yy + 0.5*dt*k2,s );
    k4 = lorenz_xdot( xx + dt*k3,yy + dt*k3,s );
    dx = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %y
    k1 = lorenz_ydot( xx,yy,zz,r );
    k2 = lorenz_ydot( xx + 0.5*dt*k1,yy + 0.5*dt*k1,zz + 0.5*dt*k1,r );
    k3 = lorenz_ydot( xx + 0.5*dt*k2,yy + 0.5*dt*k2,zz + 0.5*dt*k2,r );
    k4 = lorenz_ydot( xx + dt*k3,yy + dt*k3,zz + dt*k3,r);
    dy = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %z
    k1 = lorenz_zdot( xx,yy,zz, b );
    k2 = lorenz_zdot( xx + 0.5*dt*k1, yy + 0.5*dt*k1, zz + 0.5*dt*k1, b );
    k3 = lorenz_zdot( xx + 0.5*dt*k2,yy + 0.5*dt*k2, zz + 0.5*dt*k2, b );
    k4 = lorenz_zdot( xx + dt*k3, yy + dt*k3, zz + dt*k3, b );
    dz = (dt/6)*( k1 + 2*k2 + 2*k3 + k4 );
    
    %Update xx,yy,zz
    xx = xx + dx;
    yy = yy + dy;
    zz = zz + dz;
    
    %Store if n > N_before
    if n > N_before
        nn = n - N_before;
        x(nn,:) = xx;
        y(nn,:) = yy;
        z(nn,:) = zz;
        t(nn,:) = nn*dt;
    end
end

%%

%Lorenz ODEs
function xdot = lorenz_xdot( x,y,s )
xdot = s*( y - x );
function ydot = lorenz_ydot( x,y,z,r )
ydot = x.*(r-z) - y;
function zdot = lorenz_zdot( x,y,z,b )
zdot = x.*y - b*z;

%End of code