%pendulum
% Plots the time series and phase space (Poincare) diagram for a simple
% pendulum swinging on a frictionless, massless rod.
%
% LAST UPDATED by Andy French August 2017

function pendulum

%Number of 'exact' pendulum periods
tormax = 10;

%Timestep for numeric computation
dtor = 0.01;

%Fontsize
fsize = 18;

%Compute a selection of trajectories
theta0 = pi/12;
[theta1,tor,thetadot1] = ptraj_exact( dtor, tormax, theta0 );
theta0 = pi/6; a = 0;
[theta2,tor,thetadot2] = ptraj( dtor, tormax, theta0, a );
theta0 = pi/6; a = 1;
[theta3,tor,thetadot3] = ptraj( dtor, tormax, theta0, a );

%Plot time series
figure('name','pendulum time series','color',[1 1 1])
plot( tor, theta1, tor, theta2, tor, theta3 );
legend({'Exact','a=0','a=1'},'fontsize',fsize);
xlabel('\tau','fontsize',fsize);
ylabel('\theta /radians','fontsize',fsize);
title('Pendulum swing angle vs time','fontsize',fsize);
grid on
set(gca,'fontsize',fsize)
print( gcf,'pendulum time series.png','-r300','-dpng' )
close(gcf)

%Plot phase space (Poincare) diagram)
figure('name','pendulum phase space','color',[1 1 1])
plot( theta1, thetadot1/(2*pi), theta2, thetadot2/(2*pi), theta3, thetadot3/(2*pi) );
hold on
plot( theta1(1), thetadot1(1)/(2*pi), 'o', theta2(1), thetadot2(1)/(2*pi),'o',theta3(1), thetadot3(1)/(2*pi), 'o' );
plot( theta1(end), thetadot1(end)/(2*pi), '*', theta2(end), thetadot2(end)/(2*pi),'*',theta3(end), thetadot3(end)/(2*pi), '*' );
legend({'Exact','a=0','a=1'},'fontsize',fsize);
xlabel('\theta /radians','fontsize',fsize);
ylabel('\thetadot /2\pi','fontsize',fsize);
title('Pendulum phase space','fontsize',fsize);
grid on
axis tight
axis equal
set(gca,'fontsize',fsize)
print( gcf,'pendulum phase space.png','-r300','-dpng' )
close(gcf)

%%

%Compute pendulum trajectory assuming small angles and no friction
function [theta,tor,thetadot] = ptraj_exact( dtor, tormax, theta0 )
tor = 0:dtor:tormax;
theta = theta0*cos(2*pi*tor);
thetadot = -2*pi*theta0*sin(2*pi*tor);

%%

%Compute pendulum trajectory (including air resistance and taking into
%account large angles) using the Verlet method
function [theta,tor,thetadot] = ptraj( dtor, tormax, theta0, a )
thetadot = 0;
theta = theta0;
tor =  0:dtor:tormax;
N = numel(tor);
for n=1:N-1
    A = -4*(pi^2)*sin(theta(n)) - a*thetadot(n)*abs(thetadot(n));
    theta(n+1) = theta(n) + thetadot(n)*dtor + 0.5*A*dtor^2;
    AA = -4*(pi^2)*sin(theta(n+1)) - a*thetadot(n)*abs(thetadot(n));
    thetadot(n+1) = thetadot(n) + 0.5*dtor*( A + AA );
end

%End of code